/*  PSX Controller Decoder Library (Psx.pde)
	Written by: Kevin Ahrendt June 22nd, 2008
	
	Controller protocol implemented using Andrew J McCubbin's analysis.
	http://www.gamesx.com/controldata/psxcont/psxcont.htm
	
	Shift command is based on tutorial examples for ShiftIn and ShiftOut
	functions both written by Carlyn Maw and Tom Igoe
	http://www.arduino.cc/en/Tutorial/ShiftIn
	http://www.arduino.cc/en/Tutorial/ShiftOut

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <Psx.h>
#include <SPI.h>
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

#define WIFI

// Any pins can be used since it is done in software
#define dataPin D3
#define cmndPin D6
#define attPin D7
#define clockPin D8

#define LEDPin BUILTIN_LED

#ifdef WIFI
  int status = WL_IDLE_STATUS;
  
  //  your network SSID (name)
  const char* ssid = "ESPap";

  
  //port to listen on and send data to
  unsigned int Port = 2390;
  
  //buffer to hold incoming packet
  char packetBuffer[255];
  
  // a string to send back
  char ReplyBuffer = 'A';
  
  // IP adress of the remote host to send data to
  IPAddress remote_ip(192, 168, 4, 1);
  
  WiFiUDP Udp;
  WiFiClient remote;
#endif

// Initializes the library
Psx Psx;

// data stores the controller response
char data[6];

void setup()
{
  // Defines what each pin is used
  // (Data Pin #, Cmnd Pin #, Att Pin #, Clk Pin #, Delay)
  // Delay measures how long the clock remains at each state,
  // measured in microseconds.
  // too small delay may not work (under 5)
  Psx.setupPins(dataPin, cmndPin, attPin, clockPin, 10);

//  // Establishes LEDPin as an output so the LED can be seen
  pinMode(LEDPin, OUTPUT);
//  
//  // Turn off the LED.
  digitalWrite(LEDPin, HIGH);

  Serial.begin(115200);

#ifdef WIFI
  // Set ESP8266 in "client" mode
  WiFi.mode(WIFI_STA);

  // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
//  WiFi.begin(ssid, pass);
  WiFi.begin(ssid);

  while ( WiFi.status() != WL_CONNECTED)
  {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
       
    // wait 10 seconds for connection:
    delay(10000);
  }
  digitalWrite(BUILTIN_LED, LOW);
  Serial.println("Connected to wifi");
  printWifiStatus();

//  Udp.begin(Port);
  while (remote.connect(remote_ip, Port))
  {
    Serial.println("Wait for connection");
  }  
  Serial.println("connected");
#endif
}

void loop()
{
  // Psx.read() initiates the PSX controller and return the data
  Psx.read(data);                                      
  
#ifdef WIFI
  //Send data through UDP to remote_ip on Port
//  Udp.beginPacket(remote_ip, Port);
//  Udp.write(data,7);
//  Udp.endPacket();
//  int packetSize = Udp.parsePacket();  
//  if (packetSize)
//  {
//    // read the packet into packetBufffer
//    int len = Udp.read(packetBuffer, 255);
//    if (len > 0) packetBuffer[len] = 0;
//    if (packetBuffer[0] == ReplyBuffer)
//    {
//      //Serial.println("OK");
//    }
//  }
  remote.print(data);
#else
  Serial.print(data[0]);
  Serial.print(" ");
  Serial.print(data[1]);
  Serial.print(" ");
  Serial.print(data[2]);
  Serial.print(" ");
  Serial.print(data[3]);
  Serial.print(" ");
  Serial.print(data[4]);
  Serial.print(" ");
  Serial.print(data[5]);
  Serial.print("\n");  
#endif
  delay(20);
}

#ifdef WIFI

void printWifiStatus() {
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  // print your WiFi shield's IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
}

#endif

